/*
 * Main controllers, etc. for the kermit queue page.
 */
var module = angular.module("endroid", ["ui.bootstrap", "ngAnimate",
                                        "endroid.kermit.alert", "filterFocus",
                                        "asyncData"]);

function QueueDisplay($scope, $modal, $timeout, alertService, $http, asyncData) {
  $scope.debug = false;
  $scope.names = [];
  $scope.buglists = [];
  $scope.targets = [];
  $scope.states = ["Queued", "Running", "Aborted", "Failed", "Completed"];
  $scope.filter_shown = false;

  $scope.known_jobs = {};
  $scope.jobs = [];
  $scope.jobs_shown = 0;
  $scope.last_update = 0;

  $scope.load_filter_lists = function() {
    asyncData.getData("list?arg=user", function(data) {
      return data.user;
    }).then(function(names) {
      $scope.names = names;
    }, function() {
      alertService.add('danger', "Failed to fetch known names.",
                       5000);
    });
  };

  $scope.update_pending = false;

  $scope.lastmod_open = function($event) {
    $event.preventDefault();
    $event.stopPropagation();

    $scope.lastmod_opened = true;
  };

  $scope.state_open = function($event) {
    $event.preventDefault();
    $event.stopPropagation();

    var old_val = $scope.filters.state;
    $scope.filters.state = "";
    $scope.filters.state = old_val;
  };

  $scope.$watchCollection("filters", function() {
    console.log("Filters changed: " + $scope.filters);
    if ($scope.update_pending) {
      $timeout.cancel($scope.update_pending);
    }
    $scope.update_pending = $timeout(function() {
      console.log("Performing a (delayed) update of the jobs");
      asyncData({
        method: 'POST',
        url: 'query',
        cache: true,
        transformRequest: function(obj) {
          var str = [];
          angular.forEach(obj, function(v, k) {
            str.push(encodeURIComponent(k) + "=" + encodeURIComponent(v));
          });
          console.log(str.join('&'));
          return str.join("&");
        },
        data: $scope.filters,
        headers: {'Content-Type': 'application/x-www-form-urlencoded'}
      }).then(function(data) {
        for (var i = 0; i < data.jobs.length; i++) {
          if (!$scope.known_jobs[data.jobs[i].key]) {
            $scope.known_jobs[data.jobs[i].key] = data.jobs[i];
          }
        }
        $scope.jobs = [];
        angular.forEach($scope.known_jobs, function(val, key) {
          $scope.jobs.push(val);
        });
        $scope.jobs.reverse();
        $scope.update_pending = false;
      }, function(data) {
        alertService.add('danger', "Error fetching updated jobs list.",
                         5000);
      });
    }, 500);
  });

  var poll = function() {
      asyncData({
        method: 'POST',
        url: 'query',
        transformRequest: function(obj) {
          var str = [];
          angular.forEach(obj, function(v, k) {
            str.push(encodeURIComponent(k) + "=" + encodeURIComponent(v));
          });
          console.log(str.join('&'));
          return 'wait=' + $scope.last_update + '&' + str.join("&");
        },
        data: $scope.filters,
        headers: {'Content-Type': 'application/x-www-form-urlencoded'}
      }).then(function(data) {
        for (var i = 0; i < data.jobs.length; i++) {
          if ($scope.known_jobs[data.jobs[i].key]) {
            var old_data = $scope.known_jobs[data.jobs[i].key];
            old_data.status = data.jobs[i].status;
            old_data.status_string = data.jobs[i].status_string;
            old_data.config = data.jobs[i].config;
          } else {
            $scope.known_jobs[data.jobs[i].key] = data.jobs[i];
            $scope.jobs.splice(0, 0, data.jobs[i]);
          }
        }
        $scope.last_update = data.last_update;
        poll();
      }, function(data) {
        // Wait a bit if the connection failed. Should prob back off.
        $timeout(poll, 5000);
      });
  };

  $scope.load_filter_lists();

  $scope.filters_open = {
    username: true,
    buglist: false,
    target: false,
    state: false,
    lastmod: false
  };
  $scope.filters_focus = {
    username: false,
    buglist: false,
    target: false,
    state: false,
    lastmod: false
  };
  $scope.filters = {
    username: username,
    buglist: "",
    target: "",
    state: "",
    lastmod: new Date(new Date().getTime() - 1000 * 60 * 60 * 24 * 14)
  };

  $scope.toggle_filter = function(name, $event) {
    angular.forEach($scope.filters_open, function(v, k) {
      if (k == name) {
        $scope.filters_open[k] = !$scope.filters_open[k];
	$scope.filters_focus[k] = true;
      } else {
        $scope.filters_open[k] = false;
      }
    });
    console.log($scope.filters_focus);
  };
  $scope.filter_change = function() {
    
  };

  $scope.matches_filter = function(job) {
    var user = job.status.inputargs.username;
    var tgt = job.status.inputargs.target;
    var buglist = job.status.inputargs.buglist;
    var state = job.status_string;
    var lastmod = job.lastmod;
    var res = ((user.toLowerCase().indexOf($scope.filters.username.toLowerCase()) != -1) &&
               (tgt.toLowerCase().indexOf($scope.filters.target.toLowerCase()) != -1) &&
               (buglist.toLowerCase().indexOf($scope.filters.buglist.toLowerCase()) != -1) &&
               (state.toLowerCase().indexOf($scope.filters.state.toLowerCase()) != -1) &&
               (new Date(lastmod) > new Date($scope.filters.lastmod)));
    if (job.shown != res) {
      $scope.jobs_shown += res ? 1 : -1;
    }
    job.shown = res;
    return res;
  };

  $scope.open = function() {
    var modalInstance = $modal.open({
      templateUrl: 'submitModal.html',
      controller: ModalInstanceCtrl,
    });

    modalInstance.result.then(function(result) {
      alertService.add('success', "Submitted commit to queue as " + result,
                       5000);
    }, function(failure) {
      alertService.add('danger', "Failed to submit commit: " + failure,
                       5000);
    });
  }

  $scope.abort = function($event, key) {
    $http({
      method: "POST",
      url: "abort",
      data: 'key=' + encodeURIComponent(key),
      headers: {'Content-Type': 'application/x-www-form-urlencoded'}
    }).then(function(data) {
      console.log("Aborting " + key);
      alertService.add('warning',
                       "Aborted request " + key,
                       5000);
      $event.target.ngShow = false;
    }, function(failure) {
      alertService.add('danger',
                       'Failed to abort request ' + key + ': ' +
                       unescape(failure.data.fail), 5000);
    });
  };

  $scope.resubmitting = false;
  $scope.resubmit = function($event, key) {
    console.log(angular.element($event.target).children());
    $scope.resubmitting = key;
    $http({
      method: "POST",
      url: "submit",
      data: 'resubmitKey=' + encodeURIComponent(key),
      headers: {'Content-Type': 'application/x-www-form-urlencoded'}
    }).then(function(data) {
      console.log("Resubmitted " + key + ": " + data);
      alertService.add('success',
                       "Resubmitted request " + key + " to the queue",
                       5000);
      $event.target.ngShow = false;
      $scope.resubmitting = false;
    }, function(failure) {
      $scope.resubmitting = false;
      alertService.add('danger',
                       'Failed to resubmit request ' + key +
                       ' to the queue: ' + failure.data.fail, 5000);
    });
  };

  poll();
}

var ModalInstanceCtrl = function($scope, $modalInstance, $http, $timeout,
                                 asyncData, alertService) {
  $scope.info = {
    username: "",
    target: "",
    commitdiff: "",
    buglist: "",
    stages: [],
    ws_reset: [],
    ws_disable: [],
  };
  $scope.stages = [];
  $scope.workspaces = [];
  $scope.stage_properties = {};
  $scope.targets = [];
  asyncData.getData("list?arg=target", function(data) {
    return data.target;
  }).then(function(targets) {
    $scope.targets = targets;
  });
  $scope.$watch("info.target", function() {
    if ($scope.fetching_targets) {
      $timeout.cancel($scope.fetching_targets);
    }
    $scope.fetching_targets = $timeout(function() {
      var tgt = $scope.info.target;
      asyncData.getData("stages?target=" + encodeURIComponent(tgt))
      .then(function(data) {
        $scope.stages = data.stages;
        $scope.stage_properties = {};
        $scope.info.stages = [];
        angular.forEach(data.stages, function(v) {
          $scope.info.stages.push(v.name);
          $scope.stage_properties[v.name] = v;
        });
        console.log($scope.stage_properties);
        $scope.fetching_targets = false;
        $scope.workspaces = data.workspaces;
      }, function() {
        console.log("It appears " + tgt + " isn't a valid target");
        $scope.stages = [];
        $scope.stage_properties = {};
        $scope.info.stages = [];
        $scope.fetching_targets = false;
      });
    }, 500);
  });

  function addStage(s) {
    if ($scope.info.stages.indexOf(s) != -1) return;
    console.log("Adding stage " + s);
    if (!$scope.stage_properties[s]) {
      alertService.add("danger", "Invalid stage: " + s);
      return;
    }
    angular.forEach($scope.stage_properties[s].deps, function(d) {
      addStage(d);
    });
    $scope.info.stages.push(s);
  }
  function removeStage(s) {
    var idx = $scope.info.stages.indexOf(s);
    console.log("Removing stage " + s + " from " + idx);
    if (idx == -1) return;
    $scope.info.stages.splice(idx, 1);
    angular.forEach($scope.stage_properties, function(o) {
      console.log(o);
      if (o.deps.indexOf(s) != -1) {
        removeStage(o.name);
      }
    });
  }

  $scope.updateStages = function(stage) {
    stage = stage.getAttribute('kermit-name');
    console.log("Updating stage: " + stage);
    var idx = $scope.info.stages.indexOf(stage);
    if (idx == -1) {
      addStage(stage);
    } else {
      removeStage(stage);
    }
    console.log($scope.info.stages);
  };

  $scope.updateWorkspaces = function(button) {
    workspace = button.getAttribute('kermit-name');
    if ($scope.info.ws_reset.indexOf(workspace) != -1) {
      $scope.info.ws_reset.splice($scope.info.ws_reset.indexOf(workspace), 1);
      $scope.info.ws_disable.push(workspace);
    } else if ($scope.info.ws_disable.indexOf(workspace) != -1) {
      $scope.info.ws_disable.splice($scope.info.ws_disable.indexOf(workspace), 1);
    } else {
      $scope.info.ws_reset.push(workspace);
    }
  };

  $scope.workspaceTooltip = function(workspace) {
    if ($scope.info.ws_reset.indexOf(workspace) != -1) {
      return "Reset workspace before using it";
    } else if ($scope.info.ws_disable.indexOf(workspace) != -1) {
      return "Don't use this shared workspace";
    } else {
      return "Use this workspace as configured";
    }
  };

  $scope.ok = function() {
    $http({
      method: "POST",
      url: "submit",
      transformRequest: function(obj) {
        var str = [];
        angular.forEach(obj, function(v, k) {
          str.push(encodeURIComponent(k) + "=" + encodeURIComponent(v));
        });
        console.log(str.join('&'));
        return str.join("&");
      },
      data: $scope.info,
      headers: {'Content-Type': 'application/x-www-form-urlencoded'}
    }).then(function(data) {
      console.log(data);
      $modalInstance.close(data.data.success);
    }, function(data) {
      $modalInstance.dismiss(data.data.fail);
    });
  };

  $scope.cancel = function() {
    $modalInstance.dismiss('cancelled by you');
  };
};
